/*
 * XV thumbnail support plugin for GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <gtk/gtk.h>
#include <gmodule.h>

#include "fileutil.h"
#include "image_saver.h"
#include "intl.h"
#include "plugin.h"
#include "thumbnail_support.h"


#ifndef BUF_SIZE
#define BUF_SIZE 4096
#endif

#ifndef MAX_PATH_LEN
#define MAX_PATH_LEN 1024
#endif

/* under image directory */
#define XV_THUMNAIL_DIRECTORY ".xvpics"

static GimvImage *load_thumb (const gchar *filename,
			      const gchar *cache_type,
			      ThumbCacheSaveInfo **info);
static gboolean   save_thumb (const gchar *filename,
			      const gchar *cache_type,
			      GimvImage   *image,
			      ThumbCacheSaveInfo *info);
static gchar     *get_path   (const gchar *filename,
			      const gchar *cache_type);
static gboolean   get_size   (gint         width,
			      gint         height,
			      const gchar *cache_type,
			      gint        *width_ret,
			      gint        *height_ret);


static ThumbCacheLoader plugin_impl[] =
{
   {N_("XV thumbnail"), load_thumb, save_thumb, get_path, get_size,
    NULL, NULL, NULL, NULL},
};


PluginInfo gimv_plugin_info =
{
   type:         PLUGIN_THUMB_CACHE,
   name:         N_("XV thumbnail support"),
   version:      "0.0.1",
   author:       N_("Takuro Ashie"),
   description:  NULL,
   about:        NULL,
   config:       NULL,
   n_entries:    1,
   implement:    plugin_impl,
};


gchar *
g_module_check_init (GModule *module)
{
   return NULL;
}


void
g_module_unload (GModule *module)
{
   return;
}


static GimvImage *
load_thumb (const gchar *filename, const gchar *cache_type,
	    ThumbCacheSaveInfo **info)
{
   GimvImage *image;
   gchar *thumb_file;

   g_return_val_if_fail (filename, NULL);
   g_return_val_if_fail (cache_type, NULL);

   thumb_file = get_path (filename, cache_type);

   g_return_val_if_fail (thumb_file, NULL);

   image = gimv_image_get_xv_thumbnail (thumb_file);

   if (info) *info = NULL;

   g_free (thumb_file);

   return image;
}


static gchar *
get_path (const gchar *filename, const gchar *cache_type)
{
   const gchar *image_name;
   gchar *abspath, *image_dir, *dir;
   gchar buf[MAX_PATH_LEN];

   g_return_val_if_fail (filename, NULL);
   g_return_val_if_fail (cache_type, NULL);

   g_return_val_if_fail (!strcmp (cache_type, plugin_impl[0].label), NULL);

   abspath = fileutil_conv_rel_path_to_abs (filename);

   /* get filename */
   image_name = g_basename (abspath);
   if (!image_name) goto ERROR;

   /* get dir name */
   dir = image_dir = g_dirname (abspath);
   if (!image_dir) goto ERROR;

   g_snprintf (buf, MAX_PATH_LEN, "%s/%s/%s",
	       image_dir, XV_THUMNAIL_DIRECTORY, image_name);

   g_free (abspath);
   g_free (image_dir);
   return g_strdup (buf);

 ERROR:
   g_free (abspath);
   return NULL;
}


static gboolean
save_thumb (const gchar *filename, const gchar *cache_type,
	    GimvImage *image, ThumbCacheSaveInfo *info)
{
   GimvImage imcache;
   gchar *thumb_file;
   gint im_width = -1, im_height = -1, width = -1, height = -1;
   gboolean success, retval = FALSE;

   /* check args */
   g_return_val_if_fail (filename, FALSE);
   g_return_val_if_fail (image, FALSE);
   g_return_val_if_fail (cache_type, FALSE);

   thumb_file = get_path (filename, cache_type);
   g_return_val_if_fail (thumb_file, FALSE);

   /* get image width & height */
   gimv_image_get_size (image, &im_width, &im_height);
   if (im_width < 1 || im_height < 1) return FALSE;

   /* get thumnail width & height */
   success = get_size (im_width, im_height, cache_type, &width, &height);
   if (!success || width < 1 || height < 1) return FALSE;

   /* create cache directory if not found */
   success = mkdirs (thumb_file);
   if (!success) return FALSE;

   /* scale image */
   imcache = gimv_image_scale_and_dump (image, width, height);

   /* save cache to disk */
   if (imcache) {
      ImageSaveInfo *save_info;
      g_print (N_("save cache: %s\n"), thumb_file);
      save_info = image_saver_save_info_new (0);
      if (info)
	 save_info->cache_info = info;
      gimv_image_save_file (imcache, thumb_file, "xvpics", save_info, FALSE);
      image_saver_save_info_delete (save_info);
      gimv_image_kill (imcache);
      retval = TRUE;
   } else {
      retval = FALSE;
   }

   g_free (thumb_file);

   return retval;
}


static gboolean
get_size (gint width, gint height, const gchar *cache_type,
	  gint *width_ret, gint *height_ret)
{
   gint max_size = 96;

   /* check args */
   g_return_val_if_fail (width_ret && height_ret, FALSE);
   *width_ret = *height_ret = -1;

   g_return_val_if_fail (cache_type, FALSE);

   if (width < 1 || height < 1)
      return FALSE;

   /* no need to scale */
   if (width < max_size && height < max_size) {
      *width_ret = width;
      *height_ret = height;
      return TRUE;
   }

   /* calculate width & height of thumbnail */
   if (width > height) {
      *width_ret = max_size;
      *height_ret = (gfloat) height * (gfloat) max_size / (gfloat) width;
   } else {
      *width_ret = (gfloat) width * (gfloat) max_size / (gfloat) height;
      *height_ret = max_size;
   }

   return TRUE;
}
