/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>

#include "gimageview.h"

#include "charset.h"
#include "gimv_image.h"
#include "gtkutils.h"
#include "menu.h"
#include "prefs.h"
#include "prefs_ui_common.h"
#include "prefs_win.h"


typedef struct PrefsWin_Tag {
   /* filtering */
   GtkWidget *filter_clist;
   GtkWidget *filter_ext_entry;
   GtkWidget *filter_type_entry;
   GtkWidget *filter_disable_toggle;
   GtkWidget *filter_new;
   GtkWidget *filter_add;
   GtkWidget *filter_change;
   GtkWidget *filter_del;
   gint       filter_selected;
} PrefsWin;


typedef struct _FilterRowData
{
   gboolean enable;
   gboolean user_def;
} FilterRowData;


static PrefsWin  prefs_win;
extern Config   *config_changed;
extern Config   *config_prechanged;


static const gchar *open_window_items[] = {
   N_("Image View"),
   N_("Thumbnail View"),
   NULL
};


#if HAVE_GDK_PIXBUF
static const gchar *interpolation_items[] = {
   N_("Nearest"),
   N_("Tiles"),
   N_("Bilinear"),
   N_("Hyperbolic"),
   NULL
};
#endif /* HAVE_GDK_PIXBUF */


static const gchar *charset_to_internal_items[] = {
   N_("Never convert"),
   N_("Locale"),
   N_("Auto detect"),
   N_("Any"),
   NULL
};


static void
cb_recursive_follow_symlink (GtkWidget *radio, gpointer data)
{
   gint idx = GPOINTER_TO_INT (data);

   if (!GTK_TOGGLE_BUTTON (radio)->active) return;

   switch (idx) {
   case 0:
      config_changed->recursive_follow_link   = TRUE;
      config_changed->recursive_follow_parent = TRUE;
      break;
   case 1:
      config_changed->recursive_follow_link   = FALSE;
      config_changed->recursive_follow_parent = FALSE;
      break;
   case 2:
      config_changed->recursive_follow_link   = TRUE;
      config_changed->recursive_follow_parent = FALSE;
      break;
   default:
      break;
   }
}


static void
set_sensitive_filter (gboolean user_def)
{
   const gchar *text[2];
   gint selected = prefs_win.filter_selected;

   text[0] = gtk_entry_get_text (GTK_ENTRY (prefs_win.filter_ext_entry));
   text[1] = gtk_entry_get_text (GTK_ENTRY (prefs_win.filter_type_entry));

   if (selected < 0) {   /* no item selected */
      gtk_widget_set_sensitive (prefs_win.filter_change, FALSE);
      gtk_widget_set_sensitive (prefs_win.filter_del, FALSE);
   } else {   /* if an item selected */
      /* change button */
      gtk_widget_set_sensitive (prefs_win.filter_change, TRUE);

      /* delete button */
      gtk_widget_set_sensitive (prefs_win.filter_del, TRUE);
   }

   /* add button */
   if (!text[0] || text[0][0] == '\0') {
      if (selected < 0)
	 gtk_widget_set_sensitive (prefs_win.filter_new, FALSE);
      else
	 gtk_widget_set_sensitive (prefs_win.filter_new, TRUE);
      gtk_widget_set_sensitive (prefs_win.filter_add, FALSE);
      gtk_widget_set_sensitive (prefs_win.filter_change, FALSE);
   } else {
      gtk_widget_set_sensitive (prefs_win.filter_new, TRUE);
      gtk_widget_set_sensitive (prefs_win.filter_add, TRUE);
      if (selected >= 0)
	 gtk_widget_set_sensitive (prefs_win.filter_change, TRUE);
   }

   if (!user_def) {
      gtk_widget_set_sensitive (prefs_win.filter_ext_entry, FALSE);
      gtk_widget_set_sensitive (prefs_win.filter_type_entry, FALSE);
      gtk_widget_set_sensitive (prefs_win.filter_add, FALSE);
      gtk_widget_set_sensitive (prefs_win.filter_del, FALSE);
   } else {
      gtk_widget_set_sensitive (prefs_win.filter_ext_entry, TRUE);
      gtk_widget_set_sensitive (prefs_win.filter_type_entry, TRUE);
   }
}


static gboolean
filter_sysdef_check_enable (gchar *ext, gchar **list)
{
   gint i;

   if (!list) return TRUE;

   for (i = 0; list[i]; i++) {
      if (list[i] && *list[i] && !strcmp (ext, list[i]))
	 return FALSE;
   }

   return TRUE;
}


static gboolean
filter_check_duplicate (const gchar *ext, gint row, gboolean dialog)
{
   GtkCList *clist = GTK_CLIST (prefs_win.filter_clist);
   gboolean can_retrieve;
   gchar *text, message[BUF_SIZE];
   gint i;

   if (!ext || !*ext) return FALSE;

   for (i = 0; i < clist->rows; i++) {
      if (i == row) continue;

      can_retrieve = gtk_clist_get_text (clist, i, 0, &text);
      if (!can_retrieve) break;

      if (text && *text && !strcasecmp (ext, text)) {
	 if (dialog) {
	    g_snprintf (message, BUF_SIZE, "\"%s\" is already defined.", ext);
	    gtkutil_message_dialog ("Error!!", message);
	 }
	 return TRUE;
      }
   }

   return FALSE;
}


static void
filter_set_value ()
{
   GtkCList *clist = GTK_CLIST (prefs_win.filter_clist);
   GList *list;
   FilterRowData *rowdata;
   gchar *text[16];
   gchar **sysdef_disables = NULL, **user_defs = NULL, **sections = NULL;
   gint i, row;

   /* system defined image types */

   if (conf.imgtype_disables && *conf.imgtype_disables) {
      sysdef_disables = g_strsplit (conf.imgtype_disables, ",", -1);
      if (sysdef_disables)
	 for (i = 0; sysdef_disables[i]; i++)
	    if (*sysdef_disables[i])
	       g_strstrip (sysdef_disables[i]);
   }

   list = gimv_image_get_sysdef_ext_list_all ();

   for (i = 0; list; list = g_list_next (list), i++) {
      rowdata = g_new0 (FilterRowData, 1);
      rowdata->enable = filter_sysdef_check_enable (list->data, sysdef_disables);
      rowdata->user_def = FALSE;

      text[0] = list->data;
      text[1] = gimv_image_get_type_from_ext (text[0]);
      if (rowdata->enable)
	 text[2] = _("Enable");
      else
	 text[2] = _("Disable");
      text[3] = _("System defined");

      row = gtk_clist_append (clist, text);     
      gtk_clist_set_column_auto_resize (clist, i, TRUE);

      gtk_clist_set_row_data_full (GTK_CLIST (clist), row, (gpointer) rowdata,
				   (GtkDestroyNotify) g_free);
   }

   if (sysdef_disables)
      g_strfreev (sysdef_disables);

   /* user defined image types */

   if (conf.imgtype_user_defs && *conf.imgtype_user_defs)
      user_defs = g_strsplit (conf.imgtype_user_defs, ";", -1);
   if (!user_defs) return;

   for (i = 0; user_defs[i]; i++) {
      if (!*user_defs[i]) continue;

      sections = g_strsplit (user_defs[i], ",", 3);
      if (!sections) continue;
      g_strstrip (sections[0]);	 
      g_strstrip (sections[1]);	 
      g_strstrip (sections[2]);	 
      if (!*sections[0]) goto ERROR;
      if (filter_check_duplicate (sections[0], -1, FALSE)) goto ERROR;

      text[0] = sections[0];

      rowdata = g_new0 (FilterRowData, 1);
      rowdata->user_def = TRUE;

      if (!*sections[1])
	 text[1] = "UNKNOWN";
      else
	 text[1] = sections[1];

      if (*sections[2] && !strcasecmp (sections[2], "ENABLE")) {
	 text[2] = _("Enable");
	 rowdata->enable = TRUE;
      } else {
	 text[2] = _("Disable");
	 rowdata->enable = FALSE;
      }

      text[3] = _("User defined");

      row = gtk_clist_append (clist, text);     
      gtk_clist_set_column_auto_resize (clist, i, TRUE);

      gtk_clist_set_row_data_full (GTK_CLIST (clist), row, (gpointer) rowdata,
				   (GtkDestroyNotify) g_free);

   ERROR:
      g_strfreev (sections);
   }

   g_strfreev (user_defs);
}


static void
update_config_filter ()
{
   GtkCList *clist = GTK_CLIST (prefs_win.filter_clist);
   GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (prefs_win.filter_disable_toggle);
   FilterRowData *rowdata;
   gint i, retrieved;
   gchar *ext, *type, *disable_list = NULL, *user_defs = NULL, *tmpstr, *status;

   for (i = 0; i < clist->rows; i++) {
      rowdata = gtk_clist_get_row_data (clist, i);
      if (!rowdata) continue;

      retrieved = gtk_clist_get_text (clist, i, 0, &ext);
      if (!retrieved) continue;
      retrieved = gtk_clist_get_text (clist, i, 1, &type);
      if (!retrieved) type = "UNKNOWN";

      if (toggle->active)
	 status = "DISABLE";
      else
	 status = "ENABLE";

      if (!rowdata->user_def) {   /* system define */
	 if (rowdata->enable) continue;
	 if (!disable_list) {
	    disable_list = g_strdup (ext);
	 } else {
	    tmpstr = disable_list;
	    disable_list = g_strconcat (disable_list, ",", ext, NULL);
	    g_free (tmpstr);
	 }
      } else {                    /* user define */
	 if (!user_defs) {
	    user_defs = g_strconcat (ext, ",", type, ",", status, NULL);
	 } else {
	    tmpstr = user_defs;
	    user_defs = g_strconcat (user_defs, "; ",
				     ext, ",", type, ",", status, NULL);
	    g_free (tmpstr);
	 }
      }
   }

   if (config_changed->imgtype_disables != config_prechanged->imgtype_disables)
      g_free (config_changed->imgtype_disables);
   config_changed->imgtype_disables = disable_list;

   if (config_changed->imgtype_user_defs != config_prechanged->imgtype_user_defs)
      g_free (config_changed->imgtype_user_defs);
   config_changed->imgtype_user_defs =  user_defs;
}


static void
cb_filter_select_row (GtkCList *clist, gint row, gint col,
		     GdkEventButton *event, gpointer data)
{
   GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (prefs_win.filter_disable_toggle);
   FilterRowData *rowdata;
   gint success;
   gchar *text[2] = {NULL, NULL};

   g_return_if_fail (clist);

   prefs_win.filter_selected = row;

   success = gtk_clist_get_text (clist, row, 0, &text[0]);
   if (success && text[0])
      gtk_entry_set_text (GTK_ENTRY(prefs_win.filter_ext_entry), text[0]);

   success = gtk_clist_get_text (clist, row, 1, &text[1]);
   if (success && text[1])
      gtk_entry_set_text (GTK_ENTRY(prefs_win.filter_type_entry), text[1]);

   rowdata = gtk_clist_get_row_data (clist, row);
   if (!rowdata) return;

   gtk_toggle_button_set_active (toggle, !rowdata->enable);

   set_sensitive_filter (rowdata->user_def);
}


static void
cb_filter_unselect_row (GtkCList *clist, gint row, gint col,
		       GdkEventButton *event, gpointer data)
{
   g_return_if_fail (clist);

   prefs_win.filter_selected = -1;

   set_sensitive_filter (TRUE);
}


static void
cb_filter_ext_entry_changed (GtkEditable *entry, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.filter_clist);
   gboolean user_def = TRUE;
   gint selected = prefs_win.filter_selected;
   FilterRowData *rowdata;

   if (selected >= 0) {
      rowdata = gtk_clist_get_row_data (clist, selected);
      user_def = rowdata->user_def;
   }

   update_config_filter ();
   set_sensitive_filter (user_def);
}


static void
cb_filter_new_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.filter_clist);
   GtkEntry *entry1 = GTK_ENTRY (prefs_win.filter_ext_entry);
   GtkEntry *entry2 = GTK_ENTRY (prefs_win.filter_type_entry);

   g_return_if_fail (button);

   gtk_clist_unselect_all (clist);
   gtk_entry_set_text (entry1, "\0");
   gtk_entry_set_text (entry2, "\0");

   set_sensitive_filter (TRUE);
}


static void
cb_filter_add_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.filter_clist);
   GtkEntry *entry1 = GTK_ENTRY (prefs_win.filter_ext_entry);
   GtkEntry *entry2 = GTK_ENTRY (prefs_win.filter_type_entry);
   GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (prefs_win.filter_disable_toggle);
   FilterRowData *rowdata;
   gboolean duplicate;
   const gchar *text[16];
   gint row;

   text[0] = gtk_entry_get_text (entry1);
   text[1] = gtk_entry_get_text (entry2);

   if (!text[0] || !*text[0]) return;
   if (!text[1] || !*text[1]) text[1] = "UNKNOWN";

   duplicate = filter_check_duplicate (text[0], -1, TRUE);
   if (duplicate) return;

   rowdata = g_new0 (FilterRowData, 1);
   if (!rowdata) return;

   rowdata->user_def = TRUE;
   text[3] = _("User defined");

   if (toggle->active) {
      text[2] = _("Disable");
      rowdata->enable = FALSE;
   } else {
      text[2] = _("Enable");
      rowdata->enable = TRUE;
   }

   row = gtk_clist_append (GTK_CLIST (clist), (gchar **) text);
   gtk_clist_set_row_data_full (GTK_CLIST (clist), row, (gpointer) rowdata,
				(GtkDestroyNotify) g_free);

   gtk_entry_set_text (entry1, "\0");
   gtk_entry_set_text (entry2, "\0");

   update_config_filter ();
   set_sensitive_filter (rowdata->user_def);
}


static void
cb_filter_change_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.filter_clist);
   GtkEntry *entry1 = GTK_ENTRY (prefs_win.filter_ext_entry);
   GtkEntry *entry2 = GTK_ENTRY (prefs_win.filter_type_entry);
   GtkToggleButton *toggle = GTK_TOGGLE_BUTTON (prefs_win.filter_disable_toggle);
   FilterRowData *rowdata;
   gboolean duplicate;
   const gchar *text[16];
   gint selected = prefs_win.filter_selected;

   g_return_if_fail (button);

   if (selected < 0 && selected > clist->rows) return;

   text[0] = gtk_entry_get_text (entry1);
   text[1] = gtk_entry_get_text (entry2);

   duplicate = filter_check_duplicate (text[0], selected, TRUE);
   if (duplicate) return;

   rowdata = gtk_clist_get_row_data (clist, selected);
   if (!rowdata) return;

   if (rowdata->user_def) {
      gtk_clist_set_text (clist, selected, 0, text[0]);
      gtk_clist_set_text (clist, selected, 1, text[1]);
   }

   if (toggle->active) {
      gtk_clist_set_text (clist, selected, 2, _("Disable"));
      rowdata->enable = FALSE;
   } else {
      gtk_clist_set_text (clist, selected, 2, _("Enable"));
      rowdata->enable = TRUE;
   }

   /*
   gtk_entry_set_text (entry1, "\0");
   gtk_entry_set_text (entry2, "\0");
   */

   update_config_filter ();
   set_sensitive_filter (rowdata->user_def);
}


static void
cb_filter_del_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.filter_clist);
   FilterRowData *rowdata;
   gint selected = prefs_win.filter_selected;

   if (selected < 0 || selected > clist->rows) return;

   rowdata = gtk_clist_get_row_data (clist, selected);
   if (!rowdata->user_def) return;

   gtk_clist_remove (clist, selected);
   gtk_clist_unselect_all (clist);

   set_sensitive_filter (TRUE);
}


static void
cb_locale_charset_changed (GtkEditable *entry, gpointer data)
{
   if (config_changed->charset_locale != config_prechanged->charset_locale)
      g_free (config_changed->charset_locale);

   config_changed->charset_locale
      = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
}


static void
cb_internal_charset_changed (GtkEditable *entry, gpointer data)
{
   if (config_changed->charset_internal != config_prechanged->charset_internal)
      g_free (config_changed->charset_internal);

   config_changed->charset_internal
      = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
}


static void
cb_filename_charset_changed (GtkEditable *entry, gpointer data)
{
   if (config_changed->charset_filename != config_prechanged->charset_filename)
      g_free (config_changed->charset_filename);

   config_changed->charset_filename
      = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
}


static void
set_sensitive_filename_charset_mode (gint num, GtkWidget *combo)
{
   g_return_if_fail (combo);

   if (num == CHARSET_TO_LOCALE_ANY) {
      gtk_widget_set_sensitive (combo, TRUE);
   } else {
      gtk_widget_set_sensitive (combo, FALSE);
   }
}


static void
cb_filename_charset_conv (GtkWidget *widget, gpointer data)
{
   GtkWidget *combo = data;

   config_changed->charset_filename_mode
      = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), "num"));

   set_sensitive_filename_charset_mode (config_changed->charset_filename_mode,
					combo);
}


/*******************************************************************************
 *  prefs_common_page:
 *     @ Create common preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_common_page ()
{
   GtkWidget *main_vbox, *frame, *table, *vbox;
#if HAVE_GDK_PIXBUF
   GtkWidget *hbox;
#endif /* HAVE_GDK_PIXBUF */
   GtkWidget *label;
   GtkWidget *option_menu;
   GtkWidget *toggle, *radio0, *radio1, *radio2;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    * Default Open Window Frame
    **********************************************/
   frame = gtk_frame_new (_("Default Open Window"));
   gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
   gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, TRUE, 0);

   table = gtk_table_new (3, 2, FALSE);
   gtk_container_set_border_width(GTK_CONTAINER(table), 5);
   gtk_container_add (GTK_CONTAINER (frame), table);

   /* File Open Window Selection */
   label = gtk_label_new (_("File Open Window"));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		     GTK_EXPAND, GTK_FILL, 0, 0);
   option_menu = create_option_menu_simple (open_window_items,
					    conf.default_file_open_window,
					    &config_changed->default_file_open_window);
   gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, 0, 1,
		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

   /* Directory Open Window Selection */
   label = gtk_label_new (_("Directory Open Window"));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		     GTK_EXPAND, GTK_FILL, 0, 0);
   option_menu = create_option_menu_simple (open_window_items,
					    conf.default_dir_open_window,
					    &config_changed->default_dir_open_window);
   gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, 1, 2,
		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

   /* Archive Open Window Selection */
   label = gtk_label_new (_("Archive Open Window"));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
		     GTK_EXPAND, GTK_FILL, 0, 0);
   option_menu = create_option_menu_simple (open_window_items,
					    conf.default_arc_open_window,
					    &config_changed->default_arc_open_window);
   gtk_table_attach (GTK_TABLE (table), option_menu, 1, 2, 2, 3,
		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
   gtk_widget_set_sensitive (option_menu, FALSE);

   /**********************************************
    * Scan Directory Frame
    **********************************************/
   prefs_create_frame (_("Directory scan"), frame, vbox, main_vbox);

   /* Ignore filename extension or not */
   toggle = create_check_button (_("Scan directories recursively"),
				 conf.scan_dir_recursive,
				 cb_get_data_from_toggle,
				 &config_changed->scan_dir_recursive);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* follow symbolic link */
   radio0 = gtk_radio_button_new_with_label (NULL, _("Follow symbolic link"));
   gtk_box_pack_start (GTK_BOX (vbox), radio0, FALSE, FALSE, 0);

   radio1 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio0),
				_("Do not follow symbolic link"));
   gtk_box_pack_start (GTK_BOX (vbox), radio1, FALSE, FALSE, 0);

#ifndef USE_GTK2
   gtk_widget_set_uposition (GTK_WIDGET(radio0), 40, -1);
   gtk_widget_set_uposition (GTK_WIDGET(radio1), 40, -1);
#endif

   radio2 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio1),
                                _("Do not follow link to parent directory"));
   /* not implemented yet
   gtk_box_pack_start (GTK_BOX (vbox), radio2, FALSE, FALSE, 0);
   gtk_widget_set_uposition (GTK_WIDGET(radio2), 40, -1);
   */

   if (conf.recursive_follow_link && conf.recursive_follow_parent)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio0), TRUE);
   else if (conf.recursive_follow_link && !conf.recursive_follow_parent)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio2), TRUE);
   else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio1), TRUE);

   gtk_signal_connect (GTK_OBJECT (radio0), "toggled",
		       GTK_SIGNAL_FUNC (cb_recursive_follow_symlink),
		       GINT_TO_POINTER(0));
   gtk_signal_connect (GTK_OBJECT (radio1), "toggled",
		       GTK_SIGNAL_FUNC (cb_recursive_follow_symlink),
		       GINT_TO_POINTER(1));
   gtk_signal_connect (GTK_OBJECT (radio2), "toggled",
		       GTK_SIGNAL_FUNC (cb_recursive_follow_symlink),
		       GINT_TO_POINTER(2));


#if HAVE_GDK_PIXBUF
   /********************************************** 
    * Image Frame
    **********************************************/
   prefs_create_frame (_("Image"), frame, vbox, main_vbox);

   /* Interpolation type */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER (hbox), 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   label = gtk_label_new (_("Interpolation type for scaling"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
   option_menu = create_option_menu_simple (interpolation_items,
					    conf.interpolation,
					    (gint *) &config_changed->interpolation);
   gtk_box_pack_start (GTK_BOX (hbox), option_menu, FALSE, FALSE, 5);
#endif /* HAVE_GDK_PIXBUF */

   /********************************************** 
    * Start up Frame
    **********************************************/
   prefs_create_frame (_("Start up"), frame, vbox, main_vbox);

   /* Open thumbnail window or not */
   toggle = create_check_button (_("Open thumbnail window"),
				  conf.startup_open_thumbwin,
				  cb_get_data_from_toggle,
				  &config_changed->startup_open_thumbwin);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Scan (or not) Directory */
   toggle = create_check_button (_("Ignore directory"), !(conf.startup_read_dir),
				 cb_get_data_from_toggle_negative,
				 &config_changed->startup_read_dir);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* No warning */
   toggle = create_check_button (_("No warning"), conf.startup_no_warning,
				 cb_get_data_from_toggle,
				 &config_changed->startup_no_warning);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   return main_vbox;
}


/*******************************************************************************
 *  prefs_filter_page:
 *     @ Create filter preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_filter_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *hbox, *hbox1, *vbox;
   GtkWidget *scrollwin, *clist, *entry, *button;
   GtkWidget *toggle, *label;
   gchar *titles[] = {N_("Extension"), N_("Type"), N_("Status"), NULL};
   gint i, titles_num = sizeof (titles) / sizeof (gchar *);

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /* Ignore filename extension or not */
   toggle = create_check_button (_("Read dotfiles"),
				 conf.read_dotfile,
				 cb_get_data_from_toggle,
				 &config_changed->read_dotfile);
   gtk_box_pack_start (GTK_BOX (main_vbox), toggle, FALSE, FALSE, 0);

   /* Ignore filename extension or not */
   toggle = create_check_button (_("Ignore file name extension"),
				 !(conf.detect_filetype_by_ext),
				 cb_get_data_from_toggle_negative,
				 &config_changed->detect_filetype_by_ext);
   gtk_box_pack_start (GTK_BOX (main_vbox), toggle, FALSE, FALSE, 0);

   /********************************************** 
    * Image Types Frame
    **********************************************/
   prefs_create_frame(_("Image Types"), frame, frame_vbox, main_vbox);

   /* system defined types */
   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), scrollwin, TRUE, TRUE, 0);
   gtk_widget_set_usize (scrollwin, -1, 200);

   clist = prefs_win.filter_clist =  gtk_clist_new (4);
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   gtk_clist_set_auto_sort (GTK_CLIST (clist), TRUE);
   gtk_container_add (GTK_CONTAINER (scrollwin), clist);
   gtk_widget_show (clist);

   gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 0, TRUE);
   for (i = 0; i < titles_num; i++)
      gtk_clist_set_column_title (GTK_CLIST (clist), i, _(titles[i]));
   gtk_clist_column_titles_show (GTK_CLIST (clist));

   filter_set_value ();

   gtk_signal_connect (GTK_OBJECT (clist),"select_row",
		       GTK_SIGNAL_FUNC (cb_filter_select_row),
		       NULL);
   gtk_signal_connect (GTK_OBJECT (clist),"unselect_row",
		       GTK_SIGNAL_FUNC (cb_filter_unselect_row),
		       NULL);

   prefs_win.filter_selected = -1;

   /* entry area */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   vbox = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);

   label = gtk_label_new (_("Extension: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);

   entry = prefs_win.filter_ext_entry = gtk_entry_new ();
   gtk_widget_set_usize (entry, 100, -1);
   gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_filter_ext_entry_changed), NULL);

   vbox = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);

   label = gtk_label_new (_("Image Type: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);

   entry = prefs_win.filter_type_entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox1), entry, TRUE, TRUE, 0);

   /* disable check box */
   toggle = create_check_button (_("Disable"),
				 FALSE,
				 NULL,
				 NULL);
   prefs_win.filter_disable_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (hbox1), toggle, FALSE, FALSE, 0);

   /* edit buttons */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_end (GTK_BOX (hbox), hbox1, FALSE, TRUE, 0);
   gtk_box_set_homogeneous (GTK_BOX(hbox1), TRUE);

   button = prefs_win.filter_new = gtk_button_new_with_label (_("New"));
   gtk_widget_set_usize (button, 70, -1);
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_filter_new_button_pressed), clist);

   button = prefs_win.filter_add = gtk_button_new_with_label (_("Add"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_filter_add_button_pressed), clist);

   button = prefs_win.filter_change = gtk_button_new_with_label (_("Change"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_filter_change_button_pressed), clist);

   button = prefs_win.filter_del = gtk_button_new_with_label (_("Delete"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_filter_del_button_pressed), clist);

   return main_vbox;
}


/*******************************************************************************
 *  prefs_filter_page:
 *     @ Create filter preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_charset_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *table, *hbox;
   GtkWidget *label, *combo, *option_menu;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /********************************************** 
    * Common Frame
    **********************************************/
   prefs_create_frame (_("Common"), frame, frame_vbox, main_vbox);

   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);
   table = gtk_table_new (2, 2, FALSE);
   gtk_box_pack_start (GTK_BOX (hbox), table, FALSE, TRUE, 5);

   /* locale charset */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 0, 1,
		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
   label = gtk_label_new (_("Locale character set: "));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   combo = gtk_combo_new ();
   gtk_combo_set_popdown_strings (GTK_COMBO (combo),
				  charset_get_known_list(NULL));
   gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 0, 1,
		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
		       conf.charset_locale);
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo)->entry), "changed",
		       GTK_SIGNAL_FUNC (cb_locale_charset_changed), NULL);

   /* internal charset */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 1, 2,
		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
   label = gtk_label_new (_("Internal character set: "));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   combo = gtk_combo_new ();
   gtk_combo_set_popdown_strings (GTK_COMBO (combo),
				  charset_get_known_list(NULL));
   gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 1, 2,
		     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
		       conf.charset_internal);
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo)->entry), "changed",
		       GTK_SIGNAL_FUNC (cb_internal_charset_changed), NULL);

   /* Language to detect */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, FALSE, TRUE, 0);
   label = gtk_label_new (_("Language for auto detecting"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 5);
   option_menu = create_option_menu_simple (charset_auto_detect_labels,
					    conf.charset_auto_detect_lang,
					    &config_changed->charset_auto_detect_lang);
   gtk_box_pack_start (GTK_BOX (hbox), option_menu, FALSE, TRUE, 5);


   /********************************************** 
    * Filename Frame
    **********************************************/
   prefs_create_frame (_("File name"), frame, frame_vbox, main_vbox);

   /* charset for filename */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, FALSE, TRUE, 0);
   label = gtk_label_new (_("Character set of file name"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 5);

   /* filename charset */
   combo = gtk_combo_new ();
   gtk_widget_set_usize (combo, 120, -1);
   gtk_combo_set_popdown_strings (GTK_COMBO (combo),
				  charset_get_known_list(NULL));
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
		       conf.charset_filename);
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo)->entry), "changed",
		       GTK_SIGNAL_FUNC (cb_filename_charset_changed), NULL);

   option_menu = create_option_menu (charset_to_internal_items,
				     conf.charset_filename_mode,
				     cb_filename_charset_conv,
				     combo);

   gtk_box_pack_start (GTK_BOX (hbox), option_menu, FALSE, TRUE, 5);
   gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);

   set_sensitive_filename_charset_mode (conf.charset_filename_mode, combo);

   return main_vbox;
}


void
prefs_ui_common_set_sensitive ()
{
   set_sensitive_filter (TRUE);
}
