/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <time.h>
#include <string.h>

#include "gimageview.h"

#include "charset.h"
#include "dnd.h"
#include "gtk2-compat.h"
#include "fileutil.h"
#include "gimv_image.h"
#include "prefs.h"
#include "thumbnail.h"
#include "thumbnail_view.h"
#include "thumbview_detail.h"
#include "thumbnail_window.h"


ThumbDispMode detail_view_modes[] =
{
   {N_("Detail"),
    detailview_create,
    detailview_append_thumb_frames,
    detailview_add_thumbnail,
    detailview_refresh_thumbnail,
    detailview_redraw,
    NULL,
    detailview_adjust,
    detailview_remove_thumbview_data,
    NULL,
    detailview_set_selection},

   {N_("Detail + Icon"),
    detailview_create,
    detailview_append_thumb_frames,
    detailview_add_thumbnail,
    detailview_refresh_thumbnail,
    detailview_redraw,
    NULL,
    detailview_adjust,
    detailview_remove_thumbview_data,
    NULL,
    detailview_set_selection},

   {N_("Detail + Thumbnail"),
    detailview_create,
    detailview_append_thumb_frames,
    detailview_add_thumbnail,
    detailview_refresh_thumbnail,
    detailview_redraw,
    NULL,
    detailview_adjust,
    detailview_remove_thumbview_data,
    NULL,
    detailview_set_selection},
};
gint detail_view_modes_num = sizeof (detail_view_modes) / sizeof (ThumbDispMode);


typedef struct ThumbViewData_Tag
{
   GtkWidget   *clist;
   gint         page_pos_x[3];
   gint         page_pos_y[3];
   gchar       *dest_mode;
   gint         hilit_dir;
} ThumbViewData;


/* for converting thumbnail data to string */
typedef gchar *(*ColDataStr) (Thumbnail *thumb);

typedef struct _ListColumn {
   gchar        *title;  /* column title */
   gint          width;  /* Column width. Currentry, this value not used */
   gboolean      free;   /* Free after setting text data to columns or not */
   ColDataStr    func;   /* Pointer to function convert data to string */
} ListColumn;


/* callback functions */
static void cb_clist_button_press (GtkWidget      *widget,
				   GdkEventButton *event,
				   gpointer        data);
static void cb_click_column       (GtkWidget      *widget,
				   gint            col,
				   ThumbView      *tv);

/* for setting clist text data */
static gchar *column_data_filename (Thumbnail *thumb);
static gchar *column_data_size     (Thumbnail *thumb);
static gchar *column_data_type     (Thumbnail *thumb);
static gchar *column_data_atime    (Thumbnail *thumb);
static gchar *column_data_mtime    (Thumbnail *thumb);
static gchar *column_data_ctime    (Thumbnail *thumb);
static gchar *column_data_uid      (Thumbnail *thumb);
static gchar *column_data_gid      (Thumbnail *thumb);
static gchar *column_data_mode     (Thumbnail *thumb);
static gchar *column_data_cache    (Thumbnail *thumb);

/* other private function */
static ThumbViewData *detailview_new (ThumbView *tv);
static void detailview_set_text_data (ThumbView *tv,
				      gchar     *dest_mode,
				      GList     *start);
static void detailview_set_pixmaps   (ThumbView *tv,
				      gchar     *dest_mode);
static void store_page_pos           (ThumbView *tv);
static gint idle_reset_page_pos      (gpointer data);


static ListColumn list_columns [] =
{
   {NULL,                    0,   FALSE, NULL},
   {N_("Name"),              100, TRUE,  column_data_filename},
   {N_("Size (byte)"),       0,   TRUE,  column_data_size},
   {N_("Type"),              50,  TRUE,  column_data_type},
   {N_("Cache type"),        100, FALSE, column_data_cache},
   {N_("Access time"),       150, TRUE,  column_data_atime},
   {N_("Modification time"), 150, TRUE,  column_data_mtime},
   {N_("Change time"),       150, TRUE,  column_data_ctime},
   {N_("User"),              40,  TRUE,  column_data_uid},
   {N_("Group"),             40,  TRUE,  column_data_gid},
   {N_("Mode"),              100, TRUE,  column_data_mode},
};
static gint list_columns_num = sizeof (list_columns) / sizeof (ListColumn);


static gchar *config_columns_string = NULL;
static GList *title_idx_list        = NULL;
static gint   title_idx_list_num    = 0;
static gint   name_col              = -1;
static gint   cache_col             = -1;
static gboolean detailview_dragging            = FALSE;


/******************************************************************************
 *
 *   Callback functions.
 *
 ******************************************************************************/
static gint
cb_clist_key_press (GtkWidget *widget,
		    GdkEventKey *event,
		    ThumbView *tv)
{
   Thumbnail *thumb = NULL;
   guint keyval, popup_key = 0;
   GdkModifierType modval, popup_mod = 0;

   g_return_val_if_fail (tv, FALSE);

   keyval = event->keyval;
   modval = event->state;

   if (akey.common_popup_menu || *akey.common_popup_menu)
      gtk_accelerator_parse (akey.common_popup_menu, &popup_key, &popup_mod);

   if (GTK_CLIST (widget)->focus_row > 0)
      thumb = gtk_clist_get_row_data (GTK_CLIST (widget),
				      GTK_CLIST (widget)->focus_row);

   if (keyval == popup_key && (!popup_mod || (modval & popup_mod))) {
      thumbview_popup_menu (tv, NULL, NULL);
   } else {
      switch (keyval) {
      case GDK_Left:
      case GDK_Right:
      case GDK_Up:
      case GDK_Down:
	 return TRUE;
      case GDK_Return:
	 if (!thumb) break;
	 thumbview_set_selection_all (tv, FALSE);
	 thumbview_set_selection (thumb, TRUE);
	 thumbview_open_image (tv, thumb, 0);
	 break;
      case GDK_space:
	 break;
      case GDK_Delete:
	 thumbview_delete_files (tv);
	 break;
      default:
	 break;
      }
   }

   return FALSE;
}


static void
cb_clist_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
   ThumbView *tv = data;
   gint row;
   Thumbnail *thumb;
   gboolean clear = TRUE, found = FALSE, success;

   detailview_dragging = FALSE;

   g_return_if_fail (tv);

   gtk_widget_grab_focus (widget);

   success = gtk_clist_get_selection_info (GTK_CLIST (widget),
					   event->x, event->y,
					   &row, NULL);

   if (event->button != 4 && event->button != 5) {
      GTK_CLIST (widget)->anchor = row;
      GTK_CLIST (widget)->focus_row = row;
      gtk_widget_draw_focus (widget);
   }

   if (!success) {
      thumbview_set_selection_all (tv, FALSE);
      return;
   }

   thumb = gtk_clist_get_row_data (GTK_CLIST (widget), row);
   if (!thumb) return;

   /* set selection */
   if (event->type == GDK_BUTTON_PRESS && (event->button == 1)) {
      if (event->state & GDK_SHIFT_MASK) {
	 if (event->state & GDK_CONTROL_MASK)
	    clear = FALSE;

	 found = thumbview_set_selection_multiple (thumb, TRUE, clear);
	 if (!found)
	    thumbview_set_selection_multiple (thumb, FALSE, clear);
      } else if (!thumb->selected) {
	 if (~event->state & GDK_CONTROL_MASK)
	    thumbview_set_selection_all (tv, FALSE);
	 thumbview_set_selection (thumb, TRUE);
      } else if (thumb->selected) {
	 if (event->state & GDK_CONTROL_MASK)
	    thumbview_set_selection (thumb, FALSE);
      }
   }

   thumbview_thumb_button_press_cb (widget, event, thumb);
}


static void
cb_clist_button_release (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
   ThumbView *tv = data;
   gint row;
   Thumbnail *thumb;
   gboolean success;

   if (!tv) goto ERROR;

   success = gtk_clist_get_selection_info (GTK_CLIST (widget),
					   event->x, event->y,
					   &row, NULL);

   if (event->button != 4 && event->button != 5) {
      GTK_CLIST (widget)->anchor = row;
      GTK_CLIST (widget)->focus_row = row;
      gtk_widget_draw_focus (widget);
   }

   if (!success) {
      thumbview_set_selection_all (tv, FALSE);
      goto ERROR;
   }

   thumb = gtk_clist_get_row_data (GTK_CLIST (widget), row);
   if (!thumb) goto ERROR;

   if (event->type == GDK_BUTTON_RELEASE
       && (event->button == 1)
       && !(event->state & GDK_SHIFT_MASK)
       && !(event->state & GDK_CONTROL_MASK)
       && !detailview_dragging)
   {
      thumbview_set_selection_all (tv, FALSE);
      thumbview_set_selection (thumb, TRUE);
   }

   thumbview_thumb_button_release_cb (widget, event, thumb);

 ERROR:
   detailview_dragging = FALSE;
}


static void
cb_clist_motion_notify (GtkWidget *widget, GdkEventMotion *event, gpointer data)
{
   ThumbView *tv = data;
   gint row, on_row;
   Thumbnail *thumb;

   detailview_dragging = TRUE;

   g_return_if_fail (tv);

   on_row = gtk_clist_get_selection_info (GTK_CLIST (widget),
					  event->x, event->y, &row, NULL);
   if (!on_row) return;

   thumb = gtk_clist_get_row_data (GTK_CLIST (widget), row);
   if (!thumb) return;

   thumbview_motion_notify_cb (widget, event, thumb);
}


static void
cb_click_column (GtkWidget *widget, gint col, ThumbView *tv)
{
   ThumbWindow *tw;
   GtkWidget *sort_item;
   GList *node;
   gint idx;

   if (!tv || tv->progress) return;

   tw = tv->thumb_window;
   if (!tw) return;

   node = g_list_nth (title_idx_list, col - 1);
   if (!node) return;

   idx = GPOINTER_TO_INT (node->data);

   switch (idx) {
   case 1:
      sort_item = tw->menuitem.sort_name;
      break;
   case 2:
      sort_item = tw->menuitem.sort_size;
      break;
   case 3:
      sort_item = tw->menuitem.sort_type;
      break;
   case 5:
      sort_item = tw->menuitem.sort_access;
      break;
   case 6:
      sort_item = tw->menuitem.sort_time;
      break;
   case 7:
      sort_item = tw->menuitem.sort_change;
      break;
   default:
      return;
   }

   if (!sort_item) return;

   if (GTK_CHECK_MENU_ITEM(sort_item)->active) {
      gtk_check_menu_item_set_active
	 (GTK_CHECK_MENU_ITEM(tw->menuitem.sort_rev),
	  !GTK_CHECK_MENU_ITEM(tw->menuitem.sort_rev)->active);
   } else {
      gtk_check_menu_item_set_active
	 (GTK_CHECK_MENU_ITEM(sort_item), TRUE);
   }
}


static void
cb_select_row (GtkCList *clist, gint row, gint column,
	       GdkEventButton *event, gpointer user_data)
{
   ThumbView *tv = user_data;
   Thumbnail *thumb;
   GList *node;

   g_return_if_fail (tv);
   if (!tv->thumblist) return;

   node = g_list_nth (tv->thumblist, row);
   thumb = node->data;

   if (thumb) {
      thumb->selected = TRUE;
   }
}


static void
cb_unselect_row (GtkCList *clist, gint row, gint column,
		 GdkEventButton *event, gpointer user_data)
{
   ThumbView *tv = user_data;
   Thumbnail *thumb;
   GList *node;

   g_return_if_fail (tv);
   if (!tv->thumblist) return;

   node = g_list_nth (tv->thumblist, row);
   thumb = node->data;

   if (thumb) {
      thumb->selected = FALSE;
   }
}


#if 0
static void
cb_drag_motion (GtkWidget *clist, GdkDragContext *context,
		gint x, gint y, gint time, gpointer data)
{
   ThumbView *tv = data;
   ThumbViewData *tv_data;
   Thumbnail *thumb;
   gint row, on_row;

   g_return_if_fail (tv);

   on_row = gtk_clist_get_selection_info (GTK_CLIST (clist),
					  x, y, &row, NULL);
   if (!on_row) return;

   thumb = gtk_clist_get_row_data (GTK_CLIST (clist), row);
   if (!thumb) return;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   if (!tv_data) return;

   /* hilight row under cursor */
   if (thumb && thumbnail_is_dir (thumb) && tv_data->hilit_dir != row) {
      if (tv_data->hilit_dir != -1) {
	 gtk_clist_set_foreground (GTK_CLIST (clist), tv_data->hilit_dir,
				   &(clist->style->fg[GTK_STATE_NORMAL]));
	 gtk_clist_set_background (GTK_CLIST (clist), tv_data->hilit_dir,
				   &(clist->style->base[GTK_STATE_NORMAL]));
      }

      tv_data->hilit_dir = row;

      gtk_clist_set_foreground (GTK_CLIST (clist), tv_data->hilit_dir,
				&(clist->style->fg[GTK_STATE_SELECTED]));
      gtk_clist_set_background (GTK_CLIST (clist), tv_data->hilit_dir,
				&(clist->style->bg[GTK_STATE_SELECTED]));
   }
}


static void
cb_drag_leave (GtkWidget *clist, GdkDragContext *context, guint time,
	       gpointer data)
{
   ThumbView *tv = data;
   ThumbViewData *tv_data;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   if (!tv_data) return;

   gtk_clist_set_foreground (GTK_CLIST (clist), tv_data->hilit_dir,
			    &(clist->style->fg[GTK_STATE_NORMAL]));
   gtk_clist_set_background (GTK_CLIST (clist), tv_data->hilit_dir,
			    &(clist->style->base[GTK_STATE_NORMAL]));
   tv_data->hilit_dir = -1;
}
#endif


static void
cb_drag_data_received (GtkWidget *clist,
		       GdkDragContext *context,
		       gint x, gint y,
		       GtkSelectionData *seldata,
		       guint info,
		       guint time,
		       gpointer data)
{
#if 0
   ThumbView *tv = data;
   Thumbnail *thumb;
   gint row, on_row;

   g_return_if_fail (tv);

   on_row = gtk_clist_get_selection_info (GTK_CLIST (clist),
					  x, y, &row, NULL);
   if (!on_row) return;

   thumb = gtk_clist_get_row_data (GTK_CLIST (clist), row);
   if (!thumb) return;

   if (thumbnail_is_dir (thumb))
      tv->dnd_destdir = thumbnail_get_image_name (thumb);
#endif

   thumbview_drag_data_received_cb (clist, context, x, y,
				    seldata, info, time, data);
}



/******************************************************************************
 *
 *   private functions.
 *
 ******************************************************************************/
static ThumbViewData *
detailview_new (ThumbView *tv)
{
   ThumbViewData *tv_data;
   gint i, num;

   g_return_val_if_fail (tv, NULL);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   if (!tv_data) {
      tv_data = g_new0 (ThumbViewData, 1);
      tv_data->clist = NULL;
      num = sizeof (tv_data->page_pos_x) / sizeof (tv_data->page_pos_x[0]);
      for (i = 0; i < num; i++) {
         tv_data->page_pos_x[i] = 0;
         tv_data->page_pos_y[i] = 0;
      }
      tv_data->hilit_dir = -1;
      g_hash_table_insert (tv->disp_mode_data, DETAIL_VIEW_LABEL, tv_data);
   }

   return tv_data;
}


static gchar *
column_data_filename (Thumbnail *thumb)
{
   ThumbView *tv;

   if (!thumb) return NULL;
   tv = thumb->thumb_view;

/*
   if (tv->mode == THUMB_VIEW_MODE_DIR)
      return g_strdup (g_basename (thumb->info->filename));
   else
      return g_strdup (thumb->info->filename);
*/
   {   /********** convert charset **********/
      const gchar *filename;
      gchar *retval;

      if (tv->mode == THUMB_VIEW_MODE_DIR)
	 filename = g_basename (thumbnail_get_image_name (thumb));
      else
	 filename = thumbnail_get_image_name (thumb);

      retval = charset_to_internal (filename,
				    conf.charset_filename,
				    conf.charset_auto_detect_fn,
				    conf.charset_filename_mode);

      return retval;
   }
}


static gchar *
column_data_size (Thumbnail *thumb)
{
   if (!thumb) return NULL;

   return fileutil_size2str (thumb->info->st.st_size, FALSE);
}


static gchar *
column_data_type (Thumbnail *thumb)
{
   if (!thumb) return NULL;

   return g_strdup (gimv_image_detect_type_by_ext (thumb->info->filename));
}


static gchar *
column_data_atime (Thumbnail *thumb)
{
   if (!thumb) return NULL;

   return fileutil_time2str (thumb->info->st.st_atime);
}


static gchar *
column_data_mtime (Thumbnail *thumb)
{
   if (!thumb) return NULL;

   return fileutil_time2str (thumb->info->st.st_mtime);
}


static gchar *
column_data_ctime (Thumbnail *thumb)
{
   if (!thumb) return NULL;

   return fileutil_time2str (thumb->info->st.st_ctime);
}


static gchar *
column_data_uid (Thumbnail *thumb)
{
   if (!thumb) return NULL;

   return fileutil_uid2str (thumb->info->st.st_uid);
}


static gchar *
column_data_gid (Thumbnail *thumb)
{
   if (!thumb) return NULL;

   return fileutil_gid2str (thumb->info->st.st_gid);
}


static gchar *
column_data_mode (Thumbnail *thumb)
{
   if (!thumb) return NULL;

   return fileutil_mode2str (thumb->info->st.st_mode);
}


static gchar *
column_data_cache (Thumbnail *thumb)
{
   gchar *cache_type;

   if (!thumb) return NULL;

   cache_type = thumbnail_get_cache_type (thumb);

   return cache_type;
}


/*
 *  detailview_set_text_data:
 *     @ 
 *
 *  tv : Pointer to the ThumbView struct.
 *  dest_mode : New thumbnail view display mode.
 *  start  : Pointer to Start of the Thumbnail list.
 */
static void
detailview_set_text_data (ThumbView *tv, gchar *dest_mode, GList *start)
{
   Thumbnail *thumb;
   ThumbViewData *tv_data;
   GList *node, *data_node;
   gint i, j, start_pos, num, colnum;
   gint font_height = 16;
   gchar **image_data;

   g_return_if_fail (tv);
   if (!start) return;

   node = start;
   start_pos = g_list_position (tv->thumblist, start);
   num = g_list_length (node);
   colnum = title_idx_list_num + 1;
   image_data = g_malloc0 (sizeof (gchar *) * colnum);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   g_return_if_fail (tv_data && tv_data->clist);

   /* set text data */
   for (i = start_pos ; i < num ; i++) {
      thumb = node->data;

      image_data[0] = NULL;
      data_node = title_idx_list;
      for (j = 1; j < colnum && data_node; j++) {
	 gint idx = GPOINTER_TO_INT (data_node->data);
	 if (list_columns[idx].func) {
	    image_data[j] = list_columns[idx].func (thumb);
	 } else {
	    image_data[j] = NULL;
	 }
	 data_node = g_list_next (data_node);
      }

      gtk_clist_append (GTK_CLIST (tv_data->clist), image_data);
      gtk_clist_set_row_data (GTK_CLIST (tv_data->clist), i, thumb);

      if (thumb->selected)
	 gtk_clist_select_row (GTK_CLIST (tv_data->clist), i, -1);
      else
	 gtk_clist_unselect_row (GTK_CLIST (tv_data->clist), i, -1);

      data_node = title_idx_list;
      for (j = 1; j < colnum && data_node; j++) {
	 gint idx = GPOINTER_TO_INT (data_node->data);
	 if (list_columns[idx].free && image_data[j]) {
	    g_free (image_data[j]);
	 }
	 image_data[j] = NULL;
	 data_node = g_list_next (data_node);
      }

      if (!strcmp (DETAIL_THUMB_LABEL, dest_mode))
	 for (j = 1; j < colnum; j++) {
	    font_height = gdk_string_height (gtk_style_get_font (tv_data->clist->style),
					     "aho");
	    if (tv->ThumbnailSize > font_height)
	       gtk_clist_set_shift (GTK_CLIST (tv_data->clist), i, j,
				    (tv->ThumbnailSize - font_height) / 2, 0);
	 }
      node = g_list_next (node);
   }

   g_free (image_data);
}


/*
 *  detailview_set_pixmaps:
 *     @ set thumbnails to columns
 *
 *  tv : Pointer to the ThumbView struct.
 *  dest_mode : New thumbnail view display mode.
 */
static void
detailview_set_pixmaps (ThumbView *tv, gchar *dest_mode)
{
   Thumbnail *thumb;
   GList *node;
   gint i, num;

   if (!tv) return;

   node = g_list_first (tv->thumblist);
   num = g_list_length (node);

   /* set images */
   for ( i = 0 ; i < num ; i++ ) {
      thumb = node->data;
      detailview_add_thumbnail (thumb, dest_mode, LOAD_CACHE);
      node = g_list_next (node);
   }
}


static void
store_page_pos (ThumbView *tv)
{
   ThumbViewData *tv_data;
   GtkScrolledWindow *scrollwin;
   GtkAdjustment *hadj, *vadj;
   gchar *label;

   g_return_if_fail (tv);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   g_return_if_fail (tv_data);

   label = tv->disp_mode;
   scrollwin = GTK_SCROLLED_WINDOW (tv->container);
   hadj = gtk_scrolled_window_get_hadjustment (scrollwin);
   vadj = gtk_scrolled_window_get_vadjustment (scrollwin);

   if (!strcmp (label, DETAIL_VIEW_LABEL)) {
      tv_data->page_pos_x[0] = hadj->value;
      tv_data->page_pos_y[0] = vadj->value;
   } else if (!strcmp (label, DETAIL_ICON_LABEL)) {
      tv_data->page_pos_x[1] = hadj->value;
      tv_data->page_pos_y[1] = vadj->value;
   } else if (!strcmp (label, DETAIL_THUMB_LABEL)) {
      tv_data->page_pos_x[2] = hadj->value;
      tv_data->page_pos_y[2] = vadj->value;
   }
   /* g_print ("store: %s %f\n", label, vadj->value); */
}


static gint
idle_reset_page_pos (gpointer data)
{
   ThumbView *tv = data;
   ThumbViewData *tv_data;
   GtkScrolledWindow *scrollwin;
   GtkAdjustment *hadj, *vadj;
   gchar *label;
   gfloat page_pos_x, page_pos_y;

   g_return_val_if_fail (tv, FALSE);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   g_return_val_if_fail (tv_data, FALSE);

   label = tv_data->dest_mode;
   scrollwin = GTK_SCROLLED_WINDOW (tv->container);
   hadj = gtk_scrolled_window_get_hadjustment (scrollwin);
   vadj = gtk_scrolled_window_get_vadjustment (scrollwin);

   if (!strcmp (label, DETAIL_VIEW_LABEL)) {
      page_pos_x = tv_data->page_pos_x[0];
      page_pos_y = tv_data->page_pos_y[0];
   } else if (!strcmp (label, DETAIL_ICON_LABEL)) {
      page_pos_x = tv_data->page_pos_x[1];
      page_pos_y = tv_data->page_pos_y[1];
   } else if (!strcmp (label, DETAIL_THUMB_LABEL)) {
      page_pos_x = tv_data->page_pos_x[2];
      page_pos_y = tv_data->page_pos_y[2];
   } else {
      return FALSE;
   }

   /* g_print ("reset: %s %f\n", label, page_pos); */
   gtk_adjustment_set_value (hadj, page_pos_x);
   gtk_adjustment_set_value (vadj, page_pos_y);

   return FALSE;
}


static void
create_title_idx_list ()
{
   gchar **titles;
   gint i = 0;
   gboolean name_col_found = FALSE, cache_col_found = FALSE;

   if (!conf.detailview_columns) {
      config_columns_string = NULL;
      if (title_idx_list)
	 g_list_free (title_idx_list);
      title_idx_list_num = 0;
      name_col  = -1;
      cache_col = -1;
      return;
   }

   if (conf.detailview_columns == config_columns_string) return;

   if (title_idx_list) g_list_free (title_idx_list);
   title_idx_list = NULL;

   titles = g_strsplit (conf.detailview_columns, ",", -1);

   g_return_if_fail (titles);

   title_idx_list_num = 0;
   config_columns_string = conf.detailview_columns;

   while (titles[i]) {
      gint idx;
      idx = detailview_get_title_idx (titles[i]);
      if (!strcmp (titles[i], "Name")) {
	 name_col = i + 1;
	 name_col_found = TRUE;
      } else if (!strcmp (titles[i], "Cache type")) {
	 cache_col = i + 1;
	 cache_col_found = TRUE;
      }
      if (idx > 0) {
	 title_idx_list = g_list_append (title_idx_list, GINT_TO_POINTER (idx));
	 title_idx_list_num++;
      }
      i++;
   }

   if (!name_col_found)
      name_col = -1;
   if (!cache_col_found)
      cache_col = -1;

   g_strfreev (titles);
}



/******************************************************************************
 *
 *   public functions.
 *
 ******************************************************************************/
/*
 *  detailview_remove_thumbview_data:
 *
 *  thumb : Pointer to the Thumbnail struct.
 */
void
detailview_remove_thumbview_data (ThumbView *tv)
{
   ThumbViewData *tv_data;

   if (!tv) return;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   if (!tv_data) return;

   g_hash_table_remove (tv->disp_mode_data, DETAIL_VIEW_LABEL); 
   g_free (tv_data);
}


/*
 *  detailview_append_thumb_frames:
 *     @ add thumbnail flame (only redraw clist widget).
 *
 *  tv     : Pointer to the ThumbView struct.
 *  start  : Pointer to Start of the Thumbnail list.
 *  Return : Return FALSE if canceled by user.
 */
GList *
detailview_append_thumb_frames (ThumbView *tv, GList *start, gchar *dest_mode)
{
   GList *loadlist = NULL;

   g_return_val_if_fail (tv && start, FALSE);

   detailview_redraw (tv, dest_mode, tv->container, &loadlist);

   return loadlist;
}


/*
 *  detailview_add_thumbnail:
 *     @ append thumbnail to list.
 *
 *  thumb     : Pointer to the Thumbnail struct.
 *  dest_mode : New thumbnail view display mode.
 *  Return    : Added thumbnail pixmap widget.
 */
GtkWidget *
detailview_add_thumbnail (Thumbnail *thumb,
			  gchar *dest_mode,
			  ThumbLoadType type)
{
   ThumbView *tv;
   ThumbViewData *tv_data;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
   GtkWidget *icon = NULL, *thumbnail = NULL;
   GList *node;
   gint pos, row, col;

   g_return_val_if_fail (thumb, NULL);

   tv = thumb->thumb_view;
   g_return_val_if_fail (tv, NULL);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   g_return_val_if_fail (tv_data && tv_data->clist, NULL);

   node = g_list_find (tv->thumblist, thumb);
   pos = g_list_position (tv->thumblist, node);

   row = pos;
   col = 0;

   /*
   thumbnail = thumbnail_get_thumb (thumb);
   if (!thumbnail)
      thumbnail = create_thumbnail_auto (thumb, tv->ThumbnailSize, type);
   */

   /* set icon */
   if (!strcmp (DETAIL_ICON_LABEL, dest_mode)) {
      thumbnail_get_icon (thumb, &pixmap, &mask);
      if (pixmap) {
	 gtk_clist_set_pixmap (GTK_CLIST (tv_data->clist),
			       row, col, pixmap, mask);
	 if (cache_col >= 0)
	    gtk_clist_set_text (GTK_CLIST (tv_data->clist),
				row, cache_col, column_data_cache (thumb));
	 return icon;
      }
   }

   /* set thumbnail */
   if (!strcmp (DETAIL_THUMB_LABEL, dest_mode)) {
      thumbnail_get_thumb (thumb, &pixmap, &mask);
      if (pixmap) {
	 gtk_clist_set_pixmap (GTK_CLIST (tv_data->clist),
			       row, col, pixmap, mask);
	 if (cache_col >= 0)
	    gtk_clist_set_text (GTK_CLIST (tv_data->clist),
				row, cache_col, column_data_cache (thumb));
	 return thumbnail;
      }
   }

   return NULL;
}


/*
 *  detailview_redraw:
 *     @ remove old detailview widget and redraw.
 *
 *  tv        : Pointer to ThumbView struct for redraw.
 *  dest_mode : New thumbnail view display mode.
 *  Return    : Pointer to the new GtkTable widget.
 */
void
detailview_redraw (ThumbView *tv, gchar *dest_mode,
		   GtkWidget *scroll_win, GList **loadlist)
{
   ThumbViewData *tv_data;
   GList *node;

   g_return_if_fail (tv);

   node = g_list_find (ThumbViewList, tv);
   if (!node) return;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   if (!tv_data)
      tv_data = detailview_new (tv);

   tv_data->dest_mode = dest_mode;

   if (tv_data->clist) {
      store_page_pos (tv);
      gtk_widget_destroy (tv_data->clist);
      tv_data->clist = NULL;
   }

   if (scroll_win) {
      if (GTK_BIN (tv->container)->child)
	 gtk_widget_destroy (GTK_BIN (tv->container)->child);   
      tv_data->clist = detailview_create (tv, dest_mode);
      gtk_container_add (GTK_CONTAINER (scroll_win), tv_data->clist);
      gtk_idle_add (idle_reset_page_pos, tv);
   }

   if (!loadlist
       || !strcmp (DETAIL_VIEW_LABEL, dest_mode))
   {
       return;
   }

   /* create thumbnail load list */
   *loadlist = NULL;
   node = tv->thumblist;
   while (node) {
      Thumbnail *thumb = node->data;
      GdkPixmap *pixmap = NULL;
      GdkBitmap *mask = NULL;

      thumbnail_get_thumb (thumb, &pixmap, &mask);
      if (!pixmap)
	 *loadlist = g_list_append (*loadlist, thumb);

      node = g_list_next (node);
   }
}


/*
 *  detailview_refresh_thumbnail:
 *     @ Create thumbnail from original image and draw it.
 *
 *  thumb  : Pointer to the Thumbnail struct.
 *  Return : TRUE if success.
 */
gboolean
detailview_refresh_thumbnail (Thumbnail *thumb)
{
   GtkWidget *widget;
   ThumbView *tv;

   if (!thumb) return FALSE;

   tv = thumb->thumb_view;
   if (!tv) return FALSE;

   widget = detailview_add_thumbnail (thumb, tv->disp_mode, LOAD_CACHE);

   if (widget)
      return TRUE;
   else
      return FALSE;
}


/*
 *  detailview_adjust:
 *     @ .
 *
 *  tv     : Pointer to the ThumbView struct.
 *  tv     : Pointer to the Thumbnail struct.
 */
void
detailview_adjust (ThumbView *tv, Thumbnail *thumb)
{
   ThumbViewData *tv_data;
   GList *node;
   gint pos;

   g_return_if_fail (tv);

   node = g_list_find (ThumbViewList, tv);
   if (!node) return;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   g_return_if_fail (tv_data);

   node = g_list_find (tv->thumblist, thumb);
   if (!node) return;
   pos = g_list_position (tv->thumblist, node);

   gtk_clist_moveto (GTK_CLIST (tv_data->clist), pos, 0, 0, 0);

   return;
}


gboolean
detailview_set_selection (Thumbnail *thumb, gboolean select)
{
   ThumbView *tv;
   ThumbViewData *tv_data;
   GList *node;
   gint pos;

   g_return_val_if_fail (thumb, FALSE);

   tv = thumb->thumb_view;
   g_return_val_if_fail (tv, FALSE);
   if (g_list_length (tv->thumblist) < 1) return FALSE;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   g_return_val_if_fail (tv_data && tv_data->clist, FALSE);

   node = g_list_find (tv->thumblist, thumb);
   pos = g_list_position (tv->thumblist, node);

   if (pos >= 0) {
      thumb->selected = select;
      if (thumb->selected)
	 gtk_clist_select_row (GTK_CLIST (tv_data->clist), pos, -1);
      else
	 gtk_clist_unselect_row (GTK_CLIST (tv_data->clist), pos, -1);
   }

   return TRUE;
}


/*
 *  detailview_create:
 *     @ Create thumbnail detail view widget.
 *
 *  tv        : Pointer to the ThumbView struct.
 *  dest_mode : New display mode.
 *  Return    : GtkTable widget.
 */
GtkWidget *
detailview_create (ThumbView *tv, gchar *dest_mode)
{
   gint i, num;
   ThumbViewData *tv_data;
   GList *node;

   g_return_val_if_fail (tv, NULL);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, DETAIL_VIEW_LABEL);
   if (!tv_data) {
      tv_data = detailview_new (tv);
      g_return_val_if_fail (tv_data, NULL);
   }

   create_title_idx_list ();

   /* create clist widget */
   num = title_idx_list_num + 1;
   tv_data->clist = gtk_clist_new (num);

   if (!strcmp ("Detail + Icon", dest_mode)) {
      gtk_widget_set_name (tv_data->clist, "DetailIconMode");
   }
   if (!strcmp ("Detail + Thumbnail", dest_mode)) {
      gtk_widget_set_name (tv_data->clist, "DetailThumbMode");
   }

   gtk_clist_set_selection_mode(GTK_CLIST (tv_data->clist),
				GTK_SELECTION_MULTIPLE);
   gtk_clist_set_button_actions (GTK_CLIST (tv_data->clist), 0, GTK_BUTTON_IGNORED);
   gtk_clist_set_shadow_type (GTK_CLIST (tv_data->clist), GTK_SHADOW_OUT);
   /* gtk_clist_set_button_actions (GTK_CLIST (clist), 0, GTK_BUTTON_SELECTS); */
   gtk_widget_show (tv_data->clist);

   gtk_signal_connect_after (GTK_OBJECT (tv_data->clist), "key-press-event",
			     GTK_SIGNAL_FUNC (cb_clist_key_press), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->clist),"button_press_event",
			     GTK_SIGNAL_FUNC (cb_clist_button_press), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->clist),"button_release_event",
			     GTK_SIGNAL_FUNC (cb_clist_button_release), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist),"motion_notify_event",
		       GTK_SIGNAL_FUNC (cb_clist_motion_notify), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "click_column",
		       GTK_SIGNAL_FUNC (cb_click_column), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "select-row",
		       GTK_SIGNAL_FUNC (cb_select_row), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "unselect-row",
		       GTK_SIGNAL_FUNC (cb_unselect_row), tv);

   gtk_clist_set_use_drag_icons (GTK_CLIST (tv_data->clist), FALSE);

   /* for drop file list */
   dnd_src_set  (tv_data->clist, dnd_types, dnd_types_num);
   dnd_dest_set (tv_data->clist, dnd_types, dnd_types_num);

   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "drag_begin",
		       GTK_SIGNAL_FUNC (thumbview_drag_begin_cb), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "drag_data_get",
		       GTK_SIGNAL_FUNC (thumbview_drag_data_get_cb), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "drag_data_received",
		       GTK_SIGNAL_FUNC (cb_drag_data_received), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "drag-data-delete",
		       GTK_SIGNAL_FUNC (thumbview_drag_data_delete_cb), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "drag_end",
		       GTK_SIGNAL_FUNC (thumbview_drag_end_cb), tv);
#if 0
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "drag_motion",
		       GTK_SIGNAL_FUNC (cb_drag_motion), tv);
   gtk_signal_connect (GTK_OBJECT (tv_data->clist), "drag_leave",
		       GTK_SIGNAL_FUNC (cb_drag_leave), tv);
#endif
   gtk_object_set_data (GTK_OBJECT (tv_data->clist), "gimv-tab", tv);

   for (i = 0; i < num; i++) {
      gtk_clist_set_column_auto_resize (GTK_CLIST (tv_data->clist), i, TRUE);
   }

   /* set column titles */
   node = title_idx_list;
   for (i = 1; node; i++) {
      gint idx = GPOINTER_TO_INT (node->data);
      gtk_clist_set_column_title (GTK_CLIST (tv_data->clist),
				  i, _(list_columns[idx].title));
      node = g_list_next (node);
   }

   if (conf.detailview_show_title)
      gtk_clist_column_titles_show (GTK_CLIST (tv_data->clist));

   gtk_clist_set_column_justification(GTK_CLIST (tv_data->clist), 2, GTK_JUSTIFY_RIGHT);
   gtk_clist_set_column_justification(GTK_CLIST (tv_data->clist), 3, GTK_JUSTIFY_CENTER);

   if (!strcmp ("Detail + Icon", dest_mode)) {
      gtk_clist_set_column_width (GTK_CLIST(tv_data->clist), 0, ICON_SIZE);
      gtk_clist_set_row_height (GTK_CLIST(tv_data->clist), ICON_SIZE);
   }
   if (!strcmp ("Detail + Thumbnail", dest_mode)) {
      gtk_clist_set_column_width (GTK_CLIST (tv_data->clist), 0, tv->ThumbnailSize);
      gtk_clist_set_row_height (GTK_CLIST (tv_data->clist), tv->ThumbnailSize);
   }

   /* set data */
   detailview_set_text_data (tv, dest_mode, tv->thumblist);
   detailview_set_pixmaps (tv, dest_mode);

   return tv_data->clist;
}



/******************************************************************************
 *
 *   for preference.
 *
 ******************************************************************************/
gint
detailview_get_titles_num ()
{
   return list_columns_num;
}


gchar *
detailview_get_title (gint idx)
{
   g_return_val_if_fail (idx > 0 && idx < list_columns_num, NULL);

   return list_columns[idx].title;
}


gint
detailview_get_title_idx (const gchar *title)
{
   gint i;

   g_return_val_if_fail (title, -1);

   for (i = 1; i < list_columns_num; i++) {
      if (!list_columns[i].title) continue;
      if (!strcmp (list_columns[i].title, title))
	 return i;
   }

   return -1;
}


void
detailview_apply_config ()
{
   GList *node;

   create_title_idx_list ();

   node = ThumbViewList;

   while (node) {
      ThumbView *tv = node->data;

      if (!strcmp (tv->disp_mode, DETAIL_VIEW_LABEL)
	  || !strcmp (tv->disp_mode, DETAIL_ICON_LABEL)
	  || !strcmp (tv->disp_mode, DETAIL_THUMB_LABEL))
      {
	 thumbview_redraw (tv, tv->disp_mode, tv->container, NULL);
      }

      node = g_list_next (node);
   }
}
